#define Uses_TKeys
#define Uses_TApplication
#define Uses_TRect
#define Uses_TMenuBar
#define Uses_TSubMenu
#define Uses_TMenuItem
#define Uses_TDialog
#define Uses_TStatusLine
#define Uses_TStatusItem
#define Uses_TStatusDef
#define Uses_TDeskTop
#define Uses_TEvent
#define Uses_TProgram
#include <tv.h>

#include <bios.h>
#include "gagedemo.h"
#include "gagedlg.h"
#include "spinner.h"

///// TDemoApp  : main application class
class TDemoApp	:	public TApplication
  {
  private:
    TGageDlg         *gageDialog;                 // dialog with 3 gages
    long int         last_tick;                   // used to time idle calls
    TSpinViewer      *spinView;                   // spinner in upper left
  public :
    TDemoApp();
    static TStatusLine *initStatusLine(TRect r);  // sets up statusline
    static TMenuBar    *initMenuBar   (TRect r);  // sets up menubar
    void idle(void);                             // called when no events occur
    virtual TPalette& getPalette() const;        // decides colors and shades
    void showDialog(void);              // runs first dialog and inits control stk
  };

/////  TDemoApp: (constructor)
/////
TDemoApp::TDemoApp():
                    TProgInit(&TDemoApp::initStatusLine,
                              &TDemoApp::initMenuBar,
                              &TDemoApp::initDeskTop)
  {
  TRect r;
  r=getExtent();       // get size of screen
  r.b.y=r.a.y+1;       // set bottom one char down from top
  r.b.x=r.a.x+2;       // set right side two chars over from the left
  spinView = new TSpinViewer(r); // make spin viewer in upper left corner
  insert(spinView);              // insert him
  spinView->update();            // update him for the hell of it.

  _bios_timeofday(_TIME_GETCLOCK, &last_tick); // log the current time (see idle)
  deskTop->redraw();                   // redraw everybody, found that pallete
  menuBar->draw();                     // changes did not take effect until the
  statusLine->draw();                  // major views redraw themselves
  showDialog();                        // draw first dialog
  }

/////
///// end of TDemoApp (constructor)


/////  initStatusLine:
/////
TStatusLine* TDemoApp::initStatusLine(TRect r)
  {
  r.a.y = r.b.y-1;     // r comes in as the coordinates of the entire screen
                       // so make top y coord one up from the bottom.
                       // and pass this to the statusline constructor
  return (new TStatusLine(r,  *new TStatusDef  (0,0xFFF)  +
                              *new TStatusItem ("~Alt-X~ Exit",   kbAltX,  cmQuit)
                            )
          ); // return's )
  }
/////
///// end of TDemoApp::initStatusLine

/////  initMenuBar:
/////
TMenuBar* TDemoApp::initMenuBar(TRect r)
  {
  r.b.y = r.a.y + 1;   // r comes in as the coordinates of the entire screen
                       // so make bottom y coord one down from the top.
                       // and pass this to the TMenuBar constructor
  
  return(new TMenuBar(r,
                    *new TSubMenu("~F~ile",kbAltF)+
                    *new TMenuItem("E~x~it",cmQuit,kbAltX,hcNoContext,"Alt-X")
                    )
        );
  }
/////
///// end of TDemoApp::initMenuBar

/////  getPallete:
/////
/////       scope:  TDemoApp virtual public member function.
/////    purpose :  Looks up colors for every view in the application. Overrides
/////               TProgram::getPallete, see page 108 and 377 in Tv 2 manual.
TPalette& TDemoApp::getPalette() const
  {
  static TPalette newcolor     ( cpMyColor, sizeof( cpMyColor )-1 );
  static TPalette newblackwhite( cpMyBlackWhite, sizeof( cpMyBlackWhite )-1 );
  static TPalette newmonochrome( cpMyMonochrome, sizeof( cpMyMonochrome )-1 );
  static TPalette *palettes[] =
    {
    &newcolor,
    &newblackwhite,
    &newmonochrome
    };

  return *(palettes[appPalette]); // appPalette = TProgram data member setup
  }                               //      by initScreen during init. p 107/374
/////
///// end of getPallette

/////  idle:
/////
void TDemoApp::idle(void)
  {
  long int tick_counter;                           // init a var
  _bios_timeofday(_TIME_GETCLOCK, &tick_counter);  // get bios tick count
  if (tick_counter > last_tick)                    // see if 55 ms has passed
    {
    last_tick = tick_counter;             // if so update last_tick
    spinView->update();                   // spin the spinner
    gageDialog->update();                 // notify dialog, note: bad things
    }                                     // happen if gageDialog is destory
                                          // and this gets called.
  TApplication::idle();                   // call base class idle function.
  }
/////
///// end of TDemoApp::idle();

/////  showDialog
/////
void TDemoApp::showDialog(void)
  {
  gageDialog = new TGageDlg(TRect(15,4,65,18),"Look at the pretty gages");
  if(gageDialog)
    {
    gageDialog->insertResources();  // insertResources is my doings, not TV
    deskTop->insert(gageDialog);
    }
  }
/////
///// end of TDemoApp::showDialog()

int main()
  {
  TDemoApp demoapp;
  demoapp.run();
  return(0);
  }

